UNIT GLCD;

Interface


Procedure GLCD_cmd( b : Byte);
Procedure GLCD_dat( b : Byte);
Procedure GLCD_init;
Procedure GLCD_write( st : string);
Procedure GLCD_Test;


Implementation


Uses DOS, CRT, PUMP, DOSE50;

const

{ Command Codes for EPSON Displays }
{ COMMAND    CODE  SIZE  COMMENT   }
C_SYSTEMSET = $40; {  8 }
C_SLEEPIN   = $53; {  0 }
C_DISPON    = $59; {  1 }
C_DISPOFF   = $58; {  1 }
C_SCROLL    = $44; { 10 }
C_CSRFORM   = $5D; {  2 }
C_CGRAMADR  = $5C; {  2 }
C_CSRDIR_R  = $4C; {  0  right }
C_CSRDIR_L  = $4D; {  0  left  }
C_CSRDIR_U  = $4E; {  0  up }
C_CSRDIR_D  = $4F; {  0  down }
C_HDOTSCR   = $5A; {  1 }
C_OVLAY     = $5B; {  1 }
C_CSRW      = $46; {  2  set cursor address }
C_CSRR      = $47; {  2  read cursor address }
C_MWRITE    = $42; {  x  write to display memory }
C_MREAD     = $43; {  x  read from display memory }

M0 =  1; { 0: internal CGROM  |  1: external CGROM }
M1 =  0; { 0: 32 char CG-RAM  |  1: 64 char CG-RAM }
M2 =  1; { 0: 8x8 font        |  1: 8x16 font      }
FX =  8;
FY = 14;

SYSTEMSET : array[ 0..7] of byte =
            ( $30+M0+M1 shl 1+M2 shl 2,
              $80+(FX-1),
              $00+(FY-1),
              $27,         { C/R  }
              $39,         { TC/R }
              $EF,         { L/F  }
              $28,$00);    { APL, APH }

SCROLL    : array[ 0..9] of Byte = ( $00,$00,$EF,$B0,$04,$EF,$00,$00,$00,$00);
CSRFRM    : array[ 0..1] of Byte = ( $07,$0D);
OVR       = $01;
DISP      = $16;
HDOT      = $00;

CGRAMADR    = $7000;   { +$7FF for 8x8 font, +$FFF for 8x16 font }
Cursor_Gofs = 30*40;   { in grafics mode }
Cursor_Ginc = 40;      { increment per line = 320 div 8 = 40 }

Cursor_Tofs =  0;      { ofset in text mode }
Cursor_Tinc = 40;      { increments per line }

greetlng  = 5;
greetings : array[ 1..greetlng] of string =
            ( 'PCFG by Ulrich Prinz',
              'Thanks to:',
              #183'Uli Abromeit for the hardware',
              #183'Thorsten Erdmann for the bugfix',
              ' of the ATMEL firmware');

Grafic1 : array[ 0..7] of Byte = ( $01,$02,$04,$08,$10,$20,$40,$80);

LCD_MaxX = 319;
LCD_MaxY = 239;

type tdata32 = array[ 0..32768] of byte;
     pdata32 = ^tdata32;

var  cursor_T : word;
     cursor_G : word;

{********************************************}
{**                                        **}
{** GLCD HARDWARE INTERFACE                **}
{**                                        **}
{********************************************}


Procedure GLCD_cmd( b : Byte);
begin
  writex( $3, b);
end;

Procedure GLCD_dat( b : Byte);
begin
  writex( $2, b);
end;

Procedure GLCD_Datbl( n : byte; p : pdata32);
var i : word;
begin
  for i := 0 to n-1 do GLCD_dat( p^[i]);
end;

Procedure GLCD_Status;
begin
  ScrWinTxt( $70, 12, 1, bitstr( readx( $2)));
end;

Procedure GLCD_mwrite;
begin
  glcd_cmd( C_MWRITE);
end;

Procedure GLCD_mread;
begin
  glcd_cmd( C_MREAD);
end;

{********************************************}
{**                                        **}
{** GLCD SUPPORT ROUTINES                  **}
{**                                        **}
{********************************************}

Procedure GLCD_Tcursor( adr : word);
begin
  adr := adr+Cursor_Tofs;
  glcd_cmd( C_CSRW); glcd_dat( lo( adr)); glcd_dat( hi( adr));
  cursor_T := adr;
end;

Procedure GLCD_Gcursor( adr : word);
begin
  adr := adr+Cursor_Gofs;
  glcd_cmd( C_CSRW); glcd_dat( lo( adr)); glcd_dat( hi( adr));
  cursor_G := adr;
end;

Procedure GLCD_clrtxt;
var i : word;
begin
  glcd_TCursor(0);
  glcd_mwrite;
  for i := 0 to 30*40-1 do glcd_dat( $20);
end;

Procedure GLCD_clrgrf;
var i : word;
begin
  glcd_GCursor(0);
  glcd_mwrite;
  for i := 0 to 320*240 div 8 -1 do glcd_dat( $0);
end;

Procedure glcd_gotoxy( x, y : byte);
begin
  cursor_T := y*40+x;
  glcd_Tcursor( cursor_T);
end;

Procedure glcd_movexy( x, y : word);
begin
  cursor_G := x div 8 + y*40;
  glcd_Gcursor( cursor_g);
end;

Procedure glcd_write( st : string);
var i : word;
begin
  glcd_TCursor( Cursor_T);
  glcd_mwrite;
  for i := 1 to length( st) do glcd_dat( ord( st[i]));
  inc( Cursor_T, length( st));
end;

{********************************************}
{**                                        **}
{** GLCD ACTION                            **}
{**                                        **}
{********************************************}


Procedure GLCD_init;
begin

  { System Setup Command }
  glcd_cmd( C_SYSTEMSET);      { Start Init-Sequence                  }
  glcd_DatBl( sizeof( SYSTEMSET), @SYSTEMSET);

  glcd_cmd( C_SCROLL);
  glcd_DatBl( sizeof( SCROLL), @SCROLL);

  glcd_cmd( C_CSRFORM);
  glcd_datbl( sizeof( CSRFRM), @CSRFRM);

  glcd_cmd( C_CGRAMADR);
  glcd_dat( lo( CGRAMADR)); glcd_dat( hi( CGRAMADR));

  glcd_cmd( C_CSRDIR_R);

  glcd_cmd( C_HDOTSCR);
  glcd_dat( HDOT);

  glcd_cmd( C_OVLAY);
  glcd_dat( OVR);

  delay( 100);
  glcd_cmd( C_DISPON);
  glcd_dat( DISP);

  delay( 100);

  glcd_clrtxt;
  glcd_clrgrf;

end;

Procedure glcd_charset( fn : string);
var f   : file;
    buf : array[ 0..4096] of byte;
    i   : word;
begin

  if m2=1 then
    assign(f,fn) { 16pt }
  else
    assign(f,'fonts\test.c08');     { 8pt }

  reset(f,1);
  fillchar(buf,sizeof(buf),0);
  blockread(f,buf,sizeof(buf),i);
  close(f);

  if M0=0 then begin { internal CG-ROM + external CG-RAM 32 chars }
    glcd_TCursor(CGRAMADR+$80*8 shl m2);  {  ($80 chars later...)}
    glcd_mwrite;
    for i:=0 to 32*8 shl m2-1 do { only 32 chars }
      glcd_dat(buf[i+32*8 shl m2])
  end
  else begin { 256 chars external CG RAM }
    glcd_TCursor(CGRAMADR);
    glcd_mwrite;
    for i:=0 to 256*8 shl m2 do { 256 chars 8/16 rows (m2) }
      glcd_dat(buf[i]);

  end;
end;


Procedure GLCD_Greeting;
var i, j : byte;
    x    : word;
begin
  glcd_charset('fonts\fxdsys8.c15');
  for i := 1 to greetlng do begin
    glcd_gotoxy(0,i-1);
    glcd_mwrite;
    if M0=0 then begin
      for j := 1 to length( greetings[i]) do glcd_dat( ord( greetings[i][j])+$80);
    end
    else
      glcd_write( greetings[i]);
  end;

  glcd_movexy( 0, 0);
  glcd_mwrite;
  for x:= 1 to 40*8 do glcd_dat( $FF);
  for x:= 1 to 40*6 do glcd_dat( $FF);
end;

Procedure selectfont;
type str16 = string[16];
var sr   : searchrec;
    fils : array[ 0..30] of str16;
    filn,
    i    : byte;
    kc   : keytyp;
    c    : char;
    cp   : byte;
begin

  i := 0;
  {$I-}
  if M0=0 then FindFirst('fonts\*.C08', Archive, sr)
          else FindFirst('fonts\*.C1?', Archive, sr);
  while DosError = 0 do
  begin
    inc( i);
    fils[i] := sr.name;
    FindNext(sr);
  end;
  {$I+}
  filn := i;

  dosvar.nokeywait := false;

  InsertWindow( $2F, 8, 8, 8+16+2, 8+2+5, 1, $2F, 'FONTS');

  cp := 1;
  if filn > 0 then begin
    repeat
      scrwintxtc( $70, 10, 10, 16, fils[cp]);
      for i := 1 to 4 do begin
        if i+cp <= filn then scrwintxtc( $20, 10, 10+i, 16, fils[cp+i])
                     else scrwintxtc( $20, 10, 10+i, 16, '');
      end;

      kc := keyin( c);
      case kc of
        cudn   : begin inc( cp); if cp > filn then cp := filn; end;
        cuup   : begin dec( cp); if cp = 0 then cp := 1; end;
        RETURN : glcd_charset( 'fonts\'+fils[cp]);
      end;
    until kc = ESC;
  end;
  dosvar.nokeywait := true;
  KillWIndow;
end;

Procedure ASCIItab;
var c : byte;
begin
  glcd_gotoxy( 0, 5);
  glcd_write( '---------------ASCII Table--------------');
  for c := 0 to 255 do glcd_dat( c);
end;


Procedure ShowImage( fn : string; lcdx, lcdy : word);
type timgh = record
       Signature   : array[0..1] of char;
       FileSize    : longint;     { Filesize in bytes                   }
       reserved    : array[0..3] of char;        { not used                            }
       DataOffset  : longint;     { Offset to grafic data               }
       Size        : longint;     { Size of Info Header                 }
       Width       : longint;     { Bitmap Width                        }
       Height      : longint;     { Bitmap Height                       }
       Planes      : byte;        { Number of Planes                    }
       BitCount    : byte;        { Bits per Pixel                      }
       Compression : longint;     { Type of Compression                 }
       ImageSize   : longint;     { compressed Size of Image            }
       XpixelsPerM : longint;     { horizontal resolution: Pixels/Meter }
       YpixelsPerM : longint;     { vertical resolution: Pixels/Meter   }
       ColorsUsed  : longint;     { Number of actually used colors      }
       ColorsImportant : longint; { Number of important colors (0=all)  }
     end;
     pimgh = ^timgh;

     timg  = array[ 0..32767] of byte;
     pimg  = ^timg;


var f       : file;
    x, y    : longint;
          i : word;
    img     : pimg;
    head    : pimgh;
    imgst   : longint;
    c       : char;
    xwd     : word;
    ywd     : word;
begin
  new( img);
  head := addr( img^);

  assign( f, fn);
  reset( f, 1);
  fillchar( img^, sizeof( img^), 0);
  blockread( f, img^, sizeof( img^), i);
  close( f);

  InsertWindow( $70, 40, 5, 78, 20, 1, $70, 'BMP-Header');
  with head^ do begin
    ScrWinTxt( $70, 42,  6, 'Signature    : '+Signature[0]+Signature[1]);
    ScrWinTxt( $70, 42,  7, 'Filesize     : '+lint( filesize, 10));
    ScrWinTxt( $70, 42,  8, 'Data Offset  : '+lint( dataoffset, 10));
    ScrWinTxt( $70, 42,  4, 'Header Size  : '+lint( size, 10));
    ScrWinTxt( $70, 42, 10, 'Image Width  : '+lint( width, 10));
    ScrWinTxt( $70, 42, 11, 'Image Height : '+lint( height, 10));
    ScrWinTxt( $70, 42, 12, 'Planes       : '+ldby( planes));
    ScrWinTxt( $70, 42, 13, 'Bits/Pixel   : '+ldby( bitcount));
    ScrWinTxt( $70, 42, 14, 'Compression  : '+lint( compression, 10));
    ScrWinTxt( $70, 42, 15, 'Image Size   : '+lint( imagesize, 10));
    ScrWinTxt( $70, 42, 16, 'Horiz. Res.  : '+lint( xpixelsperm, 10));
    ScrWinTxt( $70, 42, 17, 'Vert. Res.   : '+lint( ypixelsperm, 10));
    ScrWinTxt( $70, 42, 18, 'Colors Used  : '+lint( colorsused, 10));
    ScrWinTxt( $70, 42, 19, 'Important C. : '+lint( colorsimportant, 10));

    ScrWinTxt( $70, 42, 20, 'Press any key...');
  end;


  imgst := head^.dataoffset+head^.size;
  xwd   := head^.width div 8;
  ywd   := head^.height;

  for y := ywd downto 0 do begin
    if lcdy+y <= LCD_MaxY then begin
      glcd_movexy( lcdx, lcdy+y);
      glcd_mwrite;
      for x := 0 to xwd do
        if lcdx+x <= LCD_MaxX then glcd_dat( img^[xwd*(ywd-y)+x+imgst]);
    end;
  end;
  c := readkey;

  KillWindow;
  Dispose( img);
end;


var kc   : Keytyp;
    c    : char;
    scr  : Byte;
    sdv  : Byte;

const GLCDMenu   : string = ' GLCD Menu  ~F1~ Init  ~F2~ Font  ~F3~ ASCII Table  ~F4~ ClrScr  ~F5~ Grafics  ~ESC~ Exit ';
      scrol      : Array[ 0..7] of char = ('', '', '', '', '', '', ' ', ' ');

Procedure Scroller;
begin
  dec( sdv);
  if sdv = 0 then begin
    inc( scr);
    ScrWinTxt( $72, 2, 1, scrol[scr and $03]+'          ');
    sdv := 100;
  end;
end;


Procedure GLCD_Test;
var os : string;
begin

  glcd_init;
  glcd_greeting;
  ASCIItab;

  MenuLine( $70, $74, 25, GLCDMenu);

  os := '';

  repeat
    Scroller;

    if time <> os then begin
      os := time;
      glcd_gotoxy( 40-8, 0);
      glcd_write( time);
    end;

    kc := keyin( c);
    case kc of
      F1    : begin GLCD_init; glcd_greeting; end;
      F2    : begin selectfont; glcd_greeting; end;
      F3    : begin ASCIItab; end;
      F4    : begin glcd_clrtxt; glcd_clrgrf; end;
      F5    : begin ShowImage( 'images\pump2.bmp', 0, 0); end;
    end;
  until kc = ESC;
end;

begin
  sdv := 100;
end.




