unit lcd_epson_mainu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Paracode, ComCtrls, StdCtrls, Spin, Registry, ExtCtrls;

type
  TForm1 = class(TForm)
    SpinEdit1: TSpinEdit;
    Label1: TLabel;
    CheckBox1: TCheckBox;
    Button1: TButton;
    StatusBar1: TStatusBar;
    Timer1: TTimer;
    Image1: TImage;
    Button2: TButton;
    Image2: TImage;
    FontDialog1: TFontDialog;
    Button3: TButton;
    SpinEdit2: TSpinEdit;
    Label2: TLabel;
    CheckBoxInvers: TCheckBox;
    Button4: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure CheckBoxInversClick(Sender: TObject);
  private
    { Private-Deklarationen }
    procedure TakeConfig;
    procedure OpenLPT(LPTPort:Integer);

  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;
  lcd_update:integer;

    procedure LoadConfig;
    procedure SaveConfig;

type
  LCD_Type=(LCD320x200,LCD320x240,LCD640x400,LCD640x480);

type
  // Konfigurationsdaten
  TConfig = record
    MainX,                 // Fensterpositionen
    MainY,
    CfgX,
    CfgY : integer;
    LPTPort : integer;      // Schnittstelle
    LCDTyp: LCD_Type;
    StartSymbol,           // Start als Symbol
    CompactView:Boolean;   // kompakte Darstellung
    Font:TFont;
  end { record };

{ Konfiguration }
const
  Config : TConfig =
    (MainX:-1; MainY:-1; CfgX:-1; CfgY:-1; LPTPort:1;LCDTyp:LCD320x240;
     StartSymbol:false; CompactView:false);

  Copyright = '(c) 12/99 Thomas Baumann';
  RegPath = '\Software\TBvISIOn\LCD SED1330';
  Copyright_     = 'Copyright';
  MainX_         = 'MainX';
  MainY_         = 'MainY';
  CfgX_          = 'CfgX';
  CfgY_          = 'CfgY';
  StartSymbol_   = 'StartAsSymbol';
  CompactView_   = 'CompactView';
  LPTPort_       = 'LPTPort';
  LCDTyp_        = 'LCDTyp';
  FontName_      = 'FontName';
  FontSize_      = 'FontSize';

var
  lpt : TParallelPort;
  LCD_Typ : LCD_Type;
  LCD_MaxX, LCD_MaxY : Integer;

const
  Low=False;
  High=True;
const
  // Command Codes, Bytes for Param
  C_SYSTEMSET = $40; //  8
  C_SLEEPIN   = $53; //  0
  C_DISPON    = $59; //  1
  C_DISPOFF   = $58; //  1
  C_SCROLL    = $44; // 10
  C_CSRFORM   = $5D; //  2
  C_CGRAMADR  = $5C; //  2
  C_CSRDIR_R  = $4C; //  0 right
  C_CSRDIR_L  = $4D; //  0 left
  C_CSRDIR_U  = $4E; //  0 up
  C_CSRDIR_D  = $4F; //  0 down
  C_HDOTSCR   = $5A; //  1
  C_OVLAY     = $5B; //  1
  C_CSRW      = $46; //  2 set cursor adr
  C_CSRR      = $47; //  2 read cursor adr
  C_MWRITE    = $42; //  0 write to disp mem
  C_MREAD     = $43; //  0 read from disp mem

  M0 = 0; // 0-internal CG ROM, 1-external CG-ROM/RAM
  M1 = 0; // 0 - 32 char CG-RAM, 1 - 64 char
  M2 = 1; // 0 - 8x8, 1- 8x16 matrix in CG-RAM/ROM
  FX = 8;
  FY = 16;
  // Command params
  SYSTEMSET : array[0..7] of byte =
              ($30+M0+M1 shl 1+M2 shl 2,
               $80+(FX-1),
               $00+(FY-1),
               $27, // C/R
               $30, // TC/R
               $EF, // L/F
               $28,$00); // APL, APH

  SCROLL    : array[0..9] of byte = ($00,$00,$EF,$B0,$04,$EF,$00,$00,$00,$00);
  CSRFRM    : array[0..1] of byte = ($04,$86);
  OVR       = $01;
  DISP      = $16;
  HDOT      = $00;

  CGRAMADR  = $7000; // to +$7FF for 8x8, +$FFF for 8x16

  Cursor_Gofs = 30*40; // Grafik
  Cursor_Ginc = 40; // increment per line = 320 div 8=40

  Cursor_Tofs = 0;  // text
  Cursor_Tinc = 40; // increment per line =  40

  Text1    : array[0..30] of char ='Dies ist ein Beispieltext...?!';
  Graphic1 : array[0..7] of byte = ($01,$02,$04,$08,$10,$20,$40,$80);

var
   cursor_t, cursor_g, cursor_gx, cursor_gy, cursor_tx, cursor_ty : word;
   lastsec:word;

type
  tdata32=array[0..32768] of byte;
  pdata32=^tdata32;

implementation
uses mmsystem;

{$R *.DFM}
procedure LoadConfig;
// Konfiguration laden
var
  Reg : TRegistry;

begin
  Reg:=TRegistry.Create;
  Reg.RootKey:=HKEY_CURRENT_USER;
  Reg.OpenKey(RegPath,false);
  Config.Font:=TFont.Create;

  with Config do
  begin
    try MainX:=Reg.ReadInteger(MainX_); except MainX:=-1; end;
    try MainY:=Reg.ReadInteger(MainY_); except MainY:=-1; end;
    try CfgX:=Reg.ReadInteger(CfgX_); except CfgX:=-1; end;
    try CfgY:=Reg.ReadInteger(CfgY_); except CfgY:=-1; end;
    try StartSymbol:=Reg.ReadBool(StartSymbol_); except StartSymbol:=false; end;
    try CompactView:=Reg.ReadBool(CompactView_); except CompactView:=false; end;
    try LPTPort:=Reg.ReadInteger(LPTPort_); except LPTPort:=1; end;
    try LCDTyp:=LCD_Type(Reg.ReadInteger(LCDTyp_)); except LCDTyp:=LCD320x240; end;
    try Config.Font.Name:=Reg.ReadString(FontName_); except end;
    try Config.Font.Size:=Reg.ReadInteger(FontSize_);except Font.Size:=24; Font.Name:='Arial'; end;
  end;
  Reg.Free;
end;

procedure SaveConfig;
// Konfiguration speichern
var
  Reg : TRegistry;
begin
  Reg:=TRegistry.Create;
  Reg.RootKey:=HKEY_CURRENT_USER;
  Reg.OpenKey(RegPath,true);
  with Config do
  begin
    try Reg.WriteString(Copyright_,Copyright); except; end;
    try Reg.WriteInteger(MainX_,MainX); except end;
    try Reg.WriteInteger(MainY_,MainY); except end;
    try Reg.WriteInteger(CfgX_,CfgX); except end;
    try Reg.WriteInteger(CfgY_,CfgY); except end;
    try Reg.WriteBool(StartSymbol_,StartSymbol); except; end;
    try Reg.WriteBool(CompactView_,CompactView); except; end;
    try Reg.WriteInteger(LPTPort_,LPTPort); except end;
    try Reg.WriteInteger(LCDtyp_,integer(LCDTyp)); except end;
    try Reg.WriteInteger(FontSize_,Font.Size); except end;
    try Reg.WriteString(FontName_,Font.Name); except; end;
  end;
  Reg.Free;
end;

procedure TForm1.TakeConfig;
// Konfigurationseinstellungen umsetzen
begin
  with Config do
  begin
    if MainX>=0 then Left:=MainX;
    if MainY>=0 then Top:=MainY;
  end;
end;

procedure Delay( Verz : LongInt );
var time,timelast,timebegin:longint;
begin
  timebegin := timeGetTime;
  timelast  := timebegin;
  repeat
    time:=timeGetTime;
    if (time-timelast)>100 then begin timelast := time;
      Application.ProcessMessages; { aller 100 ms } end;
  until time-timebegin >= Verz;
end;

procedure TForm1.OpenLPT(LPTPort:integer);
begin
  if LPT.openport(LPTPort) then
    StatusBar1.Panels[0].Text:='LPT'+inttostr(LPTPort)+' geffnet'
  else
    StatusBar1.Panels[0].Text:='Fehler beim ffnen von LPT'+inttostr(LPTPort);
end;



procedure A0(w:boolean);
begin
  LPT.AutoFeed:=w;
end;

procedure CS(w:boolean);
begin
  LPT.SlctIn:=w;
end;

procedure RD(w:boolean);
begin
  LPT.Initialize:=w;
end;

procedure WR(w:boolean);
begin
  LPT.Strobe:=w;
end;


procedure WR_Data(D:byte); // writes Data set in LPT.Data
begin
  LPT.Data:=D;
  CS(Low); // /CS - chipselect
  WR(Low); // /WR - write
//  delay(1);
  WR(High);
  CS(High);
end;

function RD_Data:byte; // reads Data from LPT.Data
begin
  lpt.bidi:=mode_read;
  CS(Low); // /CS - chipselect
  RD(Low); // /WR - write
//  delay(1);
  result:=LPT.Data;
  RD(High);
  CS(High);
  lpt.bidi:=mode_write;
end;

procedure comm32(c:byte);
// write a Command byte in 8080 mode (SEL1, SEL2=Gnd)
begin
  A0(High); // A0=1 - command
  WR_Data(C); // Write command set in C
end;

procedure write32(c:byte);
// write a Data Byte in 8080 mode (SEL1, SEL2=Gnd)
begin
  A0(Low); // A0=0 - data
  WR_Data(C); // Write data set in C
end;

function read32:byte;
// reads a data Byte in 8080 mode (SEL1, SEL2=Gnd)
begin
  A0(High); // A0=1 - displaydata read
  result:=RD_Data; // read data
end;

procedure data32(count:integer; p:pdata32);
var i:integer;
// puts count times data bytes out given in array pdata32
begin
  A0(Low);
  for i:=0 to count-1 do
    WR_Data(p^[i]);
end;

procedure mwrite;
begin
  comm32(C_MWRITE); A0(Low);
end;

procedure mread;
begin
  comm32(C_MREAD); A0(Low);
end;


procedure TCursor(adr:Word);
begin
  adr:=adr+Cursor_Tofs;
  comm32(C_CSRW); // Cursoradr
  write32(Lo(adr)); write32(Hi(adr));
  cursor_t:=adr;
end;

procedure Gcursor(adr:Word);
begin
  adr:=adr+Cursor_Gofs;
  comm32(C_CSRW);
  write32(Lo(adr)); write32(Hi(adr));
  cursor_g:=adr;
end;

procedure clrtext;
var i:integer;
begin
  TCursor(0);
  mwrite;
  for i:=0 to 30*40-1 do
    WR_Data($20); // space
end;

procedure clrgraphic;
var i:integer;
begin
  GCursor(0);
  mwrite;
  for i:=0 to 320*240 div 8-1 do
    WR_Data(0);

end;

procedure lcd_muster;
var muster,i:integer;
begin
  GCursor(0);

  mwrite;
  muster:=$01;
  for i:=0 to 320*240 div 8-1 do begin
    WR_Data(muster);
    if i mod 40=39 then begin
      muster:=muster shl 1; muster:=muster or ((muster shr 8) and 1);
    end;
  end;
end;

procedure LCD_gotoxy(x,y:byte);
begin
  Cursor_T:=y*40+x;
  TCursor(Cursor_T);
end;

procedure LCD_movexy(x,y:word);
begin
  Cursor_G:=x div 8+y*40;
  Cursor_Gx:=x mod 8;
  GCursor(Cursor_G);
end;

procedure LCD_write(st:string);
var i:integer;
begin
  TCursor(Cursor_T);
  mwrite;
  for i:=1 to length(st) do
    WR_Data(ord(st[i]));
  inc(Cursor_T,length(st));
end;


procedure LCD_init(LCDT:LCD_Type);
var
  i: integer;
  f:file;
  buf:array[0..4096] of byte;
  t:byte;

begin
  LCD_Typ:=LCDT;
  case LCD_typ of
    LCD320x200: begin LCD_MaxX:=319; LCD_MaxY:=199; end;
    LCD320x240: begin LCD_MaxX:=319; LCD_MaxY:=239; end;
    LCD640x400: begin LCD_MaxX:=639; LCD_MaxY:=399; end;
    LCD640x480: begin LCD_MaxX:=639; LCD_MaxY:=479; end;
  end;
  CS(High);
  WR(High);
  A0(Low);

  // SystemSet
  comm32(C_SYSTEMSET); // SS
  data32(sizeof(SYSTEMSET),@SYSTEMSET); // ss param

  comm32(C_SCROLL); // Scroll
  data32(sizeof(SCROLL),@SCROLL); // scroll param

  comm32(C_CSRFORM); //crsfrm
  data32(sizeof(CSRFRM),@CSRFRM); // csrform param

  comm32(C_CGRAMADR); // CG RAM Adr
  write32(lo(CGRAMADR)); write32(hi(CGRAMADR)); // param

  comm32(C_CSRDIR_R); //CRSDir

  comm32(C_HDOTSCR); //hdotscr
  write32(HDOT); //hdotparam

  comm32(C_OVLAY); // ovrly
  write32(OVR); //ovrly param

  comm32(C_DISPON); // disp on
  write32(DISP); // dispparam

  clrtext;

  clrgraphic;

{  Tcursor(0);
  comm32($42); // mwrite
  data32(sizeof(text1),@Text1);}
{  if M0=0 then begin
    LCD_GotoXY(0,0); LCD_Write('Zeichentabelle interner CG-ROM und');
    LCD_GotoXY(0,1); LCD_Write('32 Zeichen externer CG-RAM:');
  end
  else begin
    LCD_GotoXY(0,0); LCD_Write('Zeichentabelle mit 256 Zeichen aus');
    LCD_GotoXY(0,1); LCD_Write('externem CG-RAM.');
  end;}
  // set CGRAM

  if m2=1 then
    //assignfile(f,'bold16.ccc') // 16pt
    //assignfile(f,'test.c16') // 16pt
    assignfile(f,'fixedsys8.c15') // 16pt
  else
    //assignfile(f,'neu8.ccc'); // 8pt
   assignfile(f,'test.c08'); // 8pt
   
  reset(f,1);
  fillchar(buf,sizeof(buf),0);
  blockread(f,buf,sizeof(buf),i);
  close(f);

  if M0=0 then begin // internal CG-ROM + external CG-RAM 32 chars
    TCursor(CGRAMADR+$80*8 shl m2);  //  ($80 chars later...)
    mwrite;
    for i:=0 to 32*8 shl m2-1 do // only 32 chars
      write32(buf[i+32*8 shl m2])
  end
  else begin // 256 chars external CG RAM
    TCursor(CGRAMADR);
    mwrite;
    for i:=0 to 256*8 shl m2 do // 256 chars 8/16 rows (m2)
      write32(buf[i]);

  end;
{
  // Char table
  LCD_Gotoxy(1,0); LCD_write('0123456789ABCDEF');
  for i:=32 to 255 do begin
    if i mod 16=0 then begin
      LCD_gotoxy(0,i div 16-1);
      LCD_Write(format('%x',[i div 16]));
    end;
    write32(i);
  end;
}
{
// Readtest CG-RAM
    TCursor(CGRAMADR+$80*8 shl m2);  //  ($80 chars later...)
    mread;
    for i:=0 to 32*8 shl m2-1 do // only 32 chars
      buf[i+32*8 shl m2]:=read32;

    TCursor(CGRAMADR+$80*8 shl m2);  //  ($80 chars later...)
    mwrite;
    for i:=0 to 32*8 shl m2-1 do // only 32 chars
      write32(not(buf[i+32*8 shl m2]))

{
  // Readtest - text display memory
  LCD_GotoXY(0,0);
  mread;
  for i:=0 to 5*40-1 do
    buf[i]:=read32;
  LCD_GotoXY(0,0);
  mwrite;
  for i:=0 to 5*40-1 do
    write32(buf[i]);}

{  GCursor(200*40);
  comm32($42);
  for i:=0 to 40*8 do
    write32(i);}

{  LCD_gotoxy(0,21); LCD_Write('Hallo Welt!');}

  Form1.Timer1.Enabled:=True;
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  TakeConfig;
  lpt := TParallelPort.Create(self);
  with config do begin
    OpenLPT(LPTPort);
    SpinEdit1.Value:=LPTPort;
    CheckBOx1.Checked:=StartSymbol;
    LCD_Init(LCD_Type(LCDTyp));
  end;
  lcd_Update:=0;
end;

procedure TForm1.SpinEdit1Change(Sender: TObject);
begin
  lpt.Closeport;
  with config do begin
    LPTPort:=SpinEdit1.Value;
    OpenLPT(LPTPort);
  end;
end;

const
  dn : array[1..7] of string =
    ('Sonntag','Montag','Dienstag','Mittwoch','Donnerstag','Freitag','Samstag');


procedure TForm1.Timer1Timer(Sender: TObject);
var
  Present:TDateTime;
  Year, Month, Day, Hour, Min, Sec, MSec:word;
  x,y,i,hellschwelle:integer;
  wtst,datest,timest,st:string;
  ts:Tsize;
  buf:array[0..39] of byte;
  invers:boolean;

  procedure text(y:integer;st:string);
  var ts:tsize;
  begin
    with image2.canvas do begin
      ts:=TextExtent(st);
      TextOut(320-ts.cx, y*ts.cy, st);
      FillRect(Rect(0, y*ts.cy, 320-ts.cx, (y+1)*ts.cy));
    end;
  end;

begin
  Present:=Now;
  DecodeDate(Present, Year, Month, Day);
  DecodeTime(Present, Hour, Min, Sec, MSec);

  if Sec<>LastSec then begin
    lastSec:=sec;

    Application.Title:=format('gLCD %2d:%2.2d:%2.2d',[Hour,Min,Sec]);
    datest:=format('%2d.%2.2d.%4d',[Day,Month,Year]);
    timest:=format('%2d:%2.2d:%2.2d',[Hour,Min,Sec]);
    wtst:=dn[DayOfWeek(present)];
    //wtst:=dn[sec div 10+1];
    st:=datest+' '+timest;
    LCD_gotoxy(21,240 div FY-1); LCD_write(st);

    if M0=0 then begin
      for i:=1 to length(st) do
        st[i]:=char(ord(st[i])+$60);
      LCD_gotoxy(21,240 div FY-2); LCD_write(st);
    end;

    i:=spinedit2.value;
    hellschwelle:=i shl 16+i shl 8+i;

    with image2.canvas do begin
      Font:=Config.Font;

      Text(0,wtst);
      Text(1,datest);
      Text(2,timest);

      ts:=TextExtent(wtst);
      invers:=CheckBoxInvers.Checked;

      for y:=3*ts.cy downto lcd_update*ts.cy do begin
        lcd_movexy(0*8, 0*8 + y);
        mwrite;

        for x:=0 to 39 do begin
          buf[x]:=0;
          for i:=0 to 7 do
            if image2.Canvas.Pixels[x*8+i,y] < hellschwelle then
              buf[x]:=buf[x] or (1 shl (7-i));
        end;
        if invers then
          for x:=0 to 39 do
            write32(not(buf[x]))
        else
          for x:=0 to 39 do
            write32(buf[x]);
      end;
      //TextOut(0,-Font.Height,'88.88.8888 88:88:88');
    end;

    if Sec mod 10=9 then lcd_Update:=0 else lcd_Update:=2; //9.Sekunde ->10.Sekunde alles Updaten
  end; //sec<>lastsec
end;

procedure TForm1.Button2Click(Sender: TObject);
var x,y,i,b:integer;
begin
  timer1.Enabled:=false;
  for y:=0 to 79 do begin
    lcd_movexy(0*8, 20*8 + y);
    comm32($42); A0(Low);
    for x:=0 to 9 do begin
      b:=0;
      for i:=0 to 7 do
        b:=b or (image1.Canvas.Pixels[x*8+i,y] and 1) shl (7-i);
      write32(not(b)); // 8 Pixel out
    end;
  end;
 timer1.Enabled:=true;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  FontDialog1.Font:=config.Font;
  if FontDialog1.Execute then begin
    config.Font:=FontDialog1.Font;
    clrgraphic;
    image2.Canvas.FillRect(Rect(0,0,319,239));
  end;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  clrtext;
  clrgraphic;
  //lcd_muster;
  image2.Canvas.FillRect(Rect(0,0,319,239));
  lcd_update:=0;
end;

procedure TForm1.CheckBoxInversClick(Sender: TObject);
begin
  LCD_Update:=0;
end;

end.
